package AssociationRules;

import java.util.Collection;
import java.util.Iterator;

import associationRuleMining.AssociationRule;
import associationRuleMining.Dataset;
import associationRuleMining.Item;
import associationRuleMining.Itemset;

public class RuleManager {

	private Dataset assocDataset;
	private Collection<AssociationRule> rules;
	private int numberOfViolationFixed=0;

	public RuleManager(double[][] dataset, double minSupport,
			double minConfidence, boolean[] nominal, int maxCandidates) {
		this.assocDataset = new Dataset(dataset,nominal);
		this.rules = assocDataset.runApriori(minSupport, minConfidence, maxCandidates);
	}

	
	public void printRules(){
		this.assocDataset.printRules(this.rules, this.assocDataset);
	}
	
	/*
	 * Iterates through entries and searchs of one that violates a rule
	 */
	public double[][] correctEntries(double[][] entries) {

		int rows = entries.length;
		for(int r =0; r<rows; r++){
			for (AssociationRule aR : this.rules) {
				if(checkForViolation(entries[r],aR)){
					entries[r] = correctEntry(entries[r],aR);
					this.numberOfViolationFixed++;
				}
			}
		}
		return entries;

	}

	/**
	 * Check the entry to see if prior condition is met but the consequent is violated. 
	 * @param entry
	 * @param aR
	 * @return
	 */
	public boolean checkForViolation(double[] entry, AssociationRule aR) {
		boolean priorMet = true;
		boolean consMet =false;
		Iterator<Item> priorIterator = aR.getItemsetA().getItemIterator();
		while(priorIterator.hasNext()){
			Item item = priorIterator.next();
			if(entry[item.getCol()]!=item.getValue()){
				priorMet = false;
				break; // No need to keep going
			}
		}
		if(priorMet){
			consMet =true;
			Iterator<Item> consIterator = aR.getItemsetB().getItemIterator();
			while(consIterator.hasNext()){
				Item item = consIterator.next();
				if(entry[item.getCol()]!=item.getValue()){
					consMet = false;
					break; // No need to keep going
				}
			}
		}

		return !(priorMet==consMet);
	}

	
	public double[] correctEntry(double[] entry, AssociationRule aR){
		Iterator<Item> consIterator = aR.getItemsetB().getItemIterator();
		while(consIterator.hasNext()){
			Item item = consIterator.next();
			entry[item.getCol()]=item.getValue(); // Correct all consequents. 
				
			}
		return entry;
		}

	public int getNumberCorrected() {
		return this.numberOfViolationFixed;
	}
	
	
	public Dataset getDataSet(){
		return this.assocDataset;
	}
	}

