package ClusterDriver;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Scanner;
import java.lang.Integer;

import AssociationRules.ARulesManager;
import AssociationRules.RuleManager;
import DatabaseInterfacePackage.DatabaseManager;
import NominalToNumerical.NTN;
import Statistics.Calculator;
import WeightedKmeans.KMeans;
import WeightedKmeans.Vector;
import dummyEncodingLibrary.DummyEncoding;

public class SystemController {

	public static ArrayList<String> headers = new ArrayList<String>();

	private static String spacer ="|";

	public static void main(String[] args) throws IOException {

		for(String s : args){
			System.out.println(s);
		}

		String connectionString = args[0];
		String database = args[1];
		String username = args[2];
		String password = args[3];
		String query = args[4];
		String[] numCluster = args[5].split("~");

		String filenameWeights = args[6];
		String filenameBoolean = args[7];
		String outputlocation = args[8];
		double support = Double.parseDouble(args[9]);
		double confidence =Double.parseDouble(args[10]);
		int maxCandidates = (int)Double.parseDouble(args[11]);

		RunClustering(connectionString, database, username, password, query,
				numCluster, filenameWeights, filenameBoolean,outputlocation, support, confidence,maxCandidates);


		// java -jar PISTIS.jar jdbc:mysql://th101c-4.cs.wm.edu:33061/ priest_durbodax root s3m3rU "select * from main limit 2000;" 10~100~200~500~700~1000~1250~1500~1750 /scratch/Daniel/durbodaxWeights.dat /scratch/Daniel/durbodaxBoolean.dat /scratch/Daniel/durbodax  0.16 0.9 10000
		// java -jar PISTIS.jar jdbc:mysql://th101c-4.cs.wm.edu:33061/ openmrs root s3m3rU "select * from fullflattable limit 2000;" 10~100~200~500~700~1000~1250~1500~1750 /scratch/Daniel/openMRSWeights.dat /scratch/Daniel/openMRSBoolean.dat /scratch/Daniel/openmrs  0.16 0.9 10000




	}

	public static void RunClustering(String connectionString, String database,
			String username, String password, String query, String[] numClusterArray,
			String filenameWeights, String filenameBoolean,
			String outputLocation, double support, double confidence, int maxCandidates) throws IOException {
		long startTime;
		long endTime ;
		connectionString = connectionString+database;
		// String connectionString = "jdbc:mysql://th101c-4.cs.wm.edu:33061/"+
		// database;

		outputLocation = outputLocation+"\\";
		DatabaseManager dm = new DatabaseManager(connectionString, username,
				password);

		FileWriter fstreamSummary = null;
		try {
			fstreamSummary = new FileWriter(outputLocation+"Summary.txt");
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		BufferedWriter outSummary = new BufferedWriter(fstreamSummary);

		//entityMatrix is the original data in the database.
		String[][] entityMatrix = null;
		try {
			entityMatrix = dm.getArrary(query);
		} catch (SQLException e) {
			System.out
			.println("System was not able to create double array from query");
			e.printStackTrace();
		}

		System.out.println("Got Data Matrix for database");

		// Weights are attribute weights
		List<Double> weights = buildWeightList(filenameWeights);
		System.out.println("Read Weights");

		//encoding of an attribute is true means we are able to split the attribute into many.
		boolean[] encoding = convertListToArray(buildbooleanArray(filenameBoolean));

		//ntn normalizes the original data and has same size with the original data. 
		NTN ntn = new NTN(entityMatrix, encoding);
		System.out.println("Nominal convertion complete");
		startTime = System.currentTimeMillis();

		//Ready for the template of dummyEncoding.
		DummyEncoding dE = new DummyEncoding(ntn.nomMatrix, encoding);
		endTime = System.currentTimeMillis();
		System.out.println("DummyEncoded Matrix Time: "+(endTime-startTime));




		double[][] encodedMatrix = dE.getEncodedMatrix();
		startTime = System.currentTimeMillis();

		//Generate association rules
		//RuleManager rm = new RuleManager(ntn.nomMatrix,support,confidence,encoding,maxCandidates);

		System.out.println("Before  ARulesManager() ");
		//ARulesManager rm = new ARulesManager(ntn.nomMatrix,support,confidence,encoding,maxCandidates);

		//Temp : write normalized original data to a file  
		ARulesManager rm = new ARulesManager(ntn.nomMatrix, "D:\\matrix.txt");
		//ARulesManager rm = new ARulesManager(entityMatrix);


		//rm.GenerateRule();

		endTime = System.currentTimeMillis();
		System.out.println("Built association rules Time: "+(endTime-startTime));


		weights = dE.ExtendHeadings(weights);
		try {
			printArray(ntn.nomMatrix, outputLocation+ database
					+ "whole.csv");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		try {
			printArray(encodedMatrix, outputLocation+ database
					+ "whole.csv");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}



		for (String numCluster : numClusterArray){
			System.out.println("Working on cluster "+numCluster);
			int numberOfClusters = (int)Double.parseDouble(numCluster);
			outSummary.write("------------------------------------------------------\n");
			outSummary.write("Running Analysis on " + numberOfClusters +" clusters\n");
			outSummary.write("------------------------------------------------------\n");
			int maxRuns = 100;

			
			

			startTime = System.currentTimeMillis();
			KMeans kmeans = new KMeans(numberOfClusters, encodedMatrix, weights);
			
			
			
			Runtime r = Runtime.getRuntime();
			long t1 = r.totalMemory()/1024;
			long f1 = r.freeMemory()/1024;
			

			//clustering
			List<Integer> clusters = kmeans.partition(maxRuns);

			// removed to optimize
			// get centroids records.
			List<Vector> centroids = kmeans.calculateZ();
			
	
			
			
			
			double[][] centroidMatrix = kmeans.convertVectorArray(centroids);
			
			long t2 = r.totalMemory()/1024;      
			long f2 = r.freeMemory()/1024;  
			System.out.println(t1+"KB");    //                           9356KB
	        System.out.println(f1+"KB");    //   
			  System.out.println(t2+"KB");    //                           9356KB
		        System.out.println(f2+"KB");    //                           4917KB
			System.out.println("size of a : " + ((t2-f2)-(t1-f1))  +"kb");
			
			

			double[][] decodedMatrix = dE.decodeMatrix(centroidMatrix);

			//Apply association rules to decodedMatrix, return a new matrix with association rule
			//Temp : write normalized generated data to a file  

			//rm = new ARulesManager(decodedMatrix, "D:\\outputs\\" + numberOfClusters + "cluster-" + "matrix.txt");

			endTime = System.currentTimeMillis();
			System.out.println("Cluster and Decode Time(s): "+(endTime-startTime)/1000.0);
			
			
			System.out.println("============================================");




			String centroidOutputfile = outputLocation + numberOfClusters + "-"
					+ database + ".csv";
			String clusterLabels = outputLocation + "Labels"+numberOfClusters + "-"
					+ database + ".csv";

			//decodedMatrix = rm.correctEntries(decodedMatrix);

			try {
				printArray(encodedMatrix,
						centroidOutputfile.replace(".csv", "nominal.csv"));
			} catch (IOException e) {
				// TODO Auto-generated catch block
				System.out.println("Pistist was not able to print nominal "
						+ "	Centroid file. Please Check File Location");
			}


			try {
				//ntn.decode the numbers into data that used in the database.
				printArrayToFile(ntn.decode(decodedMatrix), centroidOutputfile);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				System.out.println("Pistist was not able to print centroidFile. "
						+ "Please Check File Location");
			}


			try {
				printIDAndClusterLabels(clusters, clusterLabels);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				System.out.println("Pistist was not able to print Centroid Label"
						+ "s. Please Check File Location");
			}

			//outSummary.write("Number of Rule Violations Corrected: "+ rm.getNumberCorrected()+"\n");
			outSummary.write("DisclosureRisk: "
					+ Calculator.calDisclosureRisk(ntn.nomMatrix, decodedMatrix,clusters)+"\n");
			outSummary.write("Unique Records: "
					+ Calculator.calUniqueRecords(ntn.nomMatrix, decodedMatrix)+"\n");
			outSummary.write("Average K members: "
					+ Calculator.calAveragemembers(clusters) + "  +/- "
					+ Calculator.calSDKmembers(clusters)+"\n");
			outSummary.write("Min K members: "
					+ Calculator.calMinKmember(clusters)+"\n");


			String [][] generatedData = ntn.decode(decodedMatrix);

			SimilarityMatrix sMatrix = new SimilarityMatrix(generatedData, entityMatrix);
			double deviation = ComputeStandardDeviation(
					centroidMatrix, encodedMatrix, clusters, weights);

			//sMatrix.printMatrix();

			System.out.println("The closure rate for " + numberOfClusters +
					"is : " +  sMatrix.avgDisclosure); 
			System.out.println("The number of unique Record for " + numberOfClusters +
					"is : " +  sMatrix.uniqueRecord); 

			System.out.println("The Standard Deviation for " + numberOfClusters +
					" is : " +  deviation); 

		}
		outSummary.close();
	}





	public static void printIDAndClusterLabels(List<Integer> clusterLabel,
			String fileName) throws IOException {
		// Create file
		FileWriter fstreamCentroids = new FileWriter(fileName);
		BufferedWriter outCentroids = new BufferedWriter(fstreamCentroids);
		int count = 0;
		for (int i : clusterLabel) {
			count++;
			outCentroids.write(i + "\n");
		}
		outCentroids.close();

	}

	public static List<Double> buildWeightList(String filename) {
		List<Double> weights = new ArrayList<Double>();

		try {
			FileInputStream fstream = new FileInputStream(filename);
			// Get the object of DataInputStream
			DataInputStream in = new DataInputStream(fstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String strLine = null;
			// Read File Line By Line
			while ((strLine = br.readLine()) != null) {

				// Print the content on the console
				weights.add(Double.parseDouble(strLine.split(",")[1]));
			}
			// Close the input stream
			in.close();
		} catch (Exception e) {// Catch exception if any
			System.err.println("Error:" + e.getMessage());
		}

		return weights;
	}

	public static List<Boolean> buildbooleanArray(String filename) {
		List<Boolean> booleanValues = new ArrayList<Boolean>();
		try {
			FileInputStream fstream = new FileInputStream(filename);
			// Get the object of DataInputStream
			DataInputStream in = new DataInputStream(fstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String strLine;
			// Read File Line By Line
			while ((strLine = br.readLine()) != null) {
				// Print the content on the console
				booleanValues.add(Boolean.parseBoolean(strLine.split(",")[1]));
				headers.add(strLine.split(",")[0]);
			}
			// Close the input stream
			in.close();
		} catch (Exception e) {// Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}

		return booleanValues;
	}

	public static boolean[] convertListToArray(List<Boolean> bools) {
		boolean[] newArray = new boolean[bools.size()];
		int count = 0;
		for (boolean b : bools) {
			newArray[count] = b;
			count++;
		}
		return newArray;
	}

	public void populateArrayFromFile(double[][] numbers, String fileName)
			throws FileNotFoundException {
		Scanner scan = new Scanner(new File(fileName));

		// Read each row.
		for (int i = 0; i < numbers.length; i++) {
			for (int j = 0; j < numbers[i].length; j++) {

				// For each number in the column, read a number
				// and put it in the array
				numbers[i][j] = scan.nextDouble();
			}
			// advance the scanner to the next line (row).
			scan.nextLine();
		}

	}

	public static double[][] generateTestArray(int rows, int cols) {
		double[][] a2 = new double[rows][cols];
		double count = 0;
		for (int r = 0; r < rows; r++) {
			for (int c = 0; c < cols; c++) {
				a2[r][c] = (int) (Math.random() * 10);
			}

		}
		return a2;
	}

	public static boolean TestArray(double[][] B, double[][] A) {
		int rows = A.length;
		int cols = A[0].length;
		for (int r = 0; r < rows; r++) {
			for (int c = 0; c < cols; c++) {
				if (A[r][c] != B[r][c]) {
					return false;
				}
			}

		}
		return true;
	}

	public static void printArray(double[][] encodedMatrix, String fileName)
			throws IOException {
		FileWriter fstreamCentroids = new FileWriter(fileName);
		BufferedWriter outCentroids = new BufferedWriter(fstreamCentroids);
		for (int i = 0; i < encodedMatrix.length; i++) {
			for (int j = 0; j < encodedMatrix[i].length; j++) {
				outCentroids.write((encodedMatrix[i][j]) + spacer);
			}
			outCentroids.write("\n");
		}
		outCentroids.close();
	}

	public static void printArrayToFile(String[][] numbers, String fileName)
			throws IOException {
		// Create file
		FileWriter fstreamCentroids = new FileWriter(fileName);
		BufferedWriter outCentroids = new BufferedWriter(fstreamCentroids);
		boolean first = true;

		int counter = 0;

		for (String label : headers) {
			if (!first) {
				outCentroids.write(spacer);
			} else {
				first = false;
			}
			outCentroids.write(label);
			counter++;

		}

		System.out.println("***counter = " + counter);
		outCentroids.write("\n");

		for (int i = 0; i < numbers.length; i++) {
			for (int j = 0; j < numbers[i].length; j++) {
				outCentroids.write((numbers[i][j]) + spacer);
			}
			outCentroids.write("\n");
		}
		// Close the output stream
		outCentroids.close();
	}

	public static double ComputeStandardDeviation(double[][] centroidM, 
			double[][] encodeM, List<Integer> cluster, List<Double> weights){

		double result = 0;
		double[] distSum = new double[centroidM.length];
		double[]   distNum = new double[centroidM.length];


		if (cluster.size() != encodeM.length)
			throw new IllegalArgumentException();

		int j = 0;
		for(int i : cluster)
		{
			double distance = euclideanNorm(encodeM[j], centroidM[i], weights);
			distSum[i] = distSum[i] + distance;
			distNum[i] =  distNum[i] + 1;
			j++;
		}

		//ArrayList<Double>  sd = new ArrayList<Double>(centroidM.length);

		double sumSD = 0;

		int i = 0;
		while(i < distSum.length)
		{
			sumSD += Math.sqrt(distSum[i]/distNum[i]);
			i++;
		}
		result = sumSD/distSum.length;

		return result;
	}

	//	public static double ComputeStandardDeviation(double[][] centroidM, 
	//			double[][] encodeM, List<Integer> cluster, List<Double> weights){
	//
	//		double result = 0;
	//		ArrayList<Double>  distSum = new ArrayList<Double>(centroidM.length);
	//		ArrayList<Double>  distNum = new ArrayList<Double>(centroidM.length);
	//		
	//		
	//		if (cluster.size() != encodeM.length)
	//			throw new IllegalArgumentException();
	//
	//		for(int i : cluster)
	//		{
	//			double distance = euclideanNorm(encodeM[i], centroidM[cluster.get(i)], weights);
	//			double newSum = distSum.get(cluster.get(i)) + distance;
	//			distSum.set(cluster.get(i), newSum);
	//			distNum.set(cluster.get(i), distNum.get(cluster.get(i))+1);
	//		}
	//
	//		//ArrayList<Double>  sd = new ArrayList<Double>(centroidM.length);
	//		
	//		double sumSD = 0;
	//		
	//		int i = 0;
	//		while(i < distSum.size())
	//		{
	//			sumSD += Math.sqrt(distSum.get(i)/distNum.get(i));
	//			i++;
	//		}
	//		result = sumSD/distSum.size();
	//		
	//		return result;
	//	}



	public static double euclideanNorm(double[] first, double[] second, List<Double> weightList) {
		if (first.length != first.length)
			throw new IllegalArgumentException();

		double sumOfSquares = 0;
		for (int i = 0; i < first.length; i++) {
			sumOfSquares 
			+= weightList.get(i)*(first[i] - second[i]) * (first[i] - second[i]);  
		}

		return Math.sqrt(sumOfSquares);
	}






}
