package WeightedKmeans;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * @author Ahmad
 */
public class Vector {

        private double EPSILON = 0.000000001;
        
        private double[] values;
        
        private String name = "un-named";
        
        public Vector(double... values) {
                this.values = values;
        }
        
        public Vector(String name, double... values) {
                this.name = name;
                this.values = values;
        }
        
        public static double euclideanNorm(Vector first, Vector second, List<Double> weightList) {
                if (first.values.length != second.values.length)
                        throw new IllegalArgumentException();
                
                double sumOfSquares = 0;
                for (int i = 0; i < first.values.length; i++) {
                        sumOfSquares 
                                += weightList.get(i)*(first.values[i] - second.values[i]) * (first.values[i] - second.values[i]);  
                }
                
                return Math.sqrt(sumOfSquares);
        }
        
        @Override
        public boolean equals(Object obj) {
                Vector other = (Vector) obj;
                for (int i = 0; i < values.length; i++) {
                        if (Math.abs((values[i] - other.values[i])) > EPSILON)
                                return  false;
                }
                
                return true;
        }
        
        @Override
        public String toString() {
                StringBuilder sb = new StringBuilder("");
                for (int i = 0; i < values.length; i++) {
                        sb.append(values[i]);
                        if (i < values.length - 1)
                                sb.append(", ");
                }
                
                
                return sb.append("").toString();
        }
   
        
        /**
         * 
         * @param points - list of vectors prepresenting each pattern. 
         * @param weightList - list of weightes of each attribute of the vector
         * @return
         */
        
        public int getNearestPointIndex(List<Vector> points, List<Double> weightList) {
                if (points == null || points.size() == 0)
                        throw new IllegalArgumentException();
                
                int nearestVectorIndex = 0;
                double minDistance = euclideanNorm(this, points.get(0),weightList);
                for (int i = 1; i < points.size(); i++) {
                        double distance = euclideanNorm(this, points.get(i),weightList);
                        if (distance < minDistance) {
                                nearestVectorIndex = i;
                                minDistance = distance;
                        }
                }
                
                return nearestVectorIndex;
        }
        
        public double getValueAtIndex(int index){
        	return values[index];
        }
        /**
         * This function is used for replace 
         */
        public void replaceValueAtIndex(int index, List<Double> repList ){
        	double[] newValues = new double[values.length+repList.size()];
        	for(int beforeIndex =0; beforeIndex < index; beforeIndex++){
        		newValues[beforeIndex] = values[beforeIndex];
        	}
        	for(int inBetweenIndex =0; inBetweenIndex<repList.size(); inBetweenIndex++){
        		newValues[index+inBetweenIndex] = repList.get(inBetweenIndex);
        	}
        	for(int AfterIndex = index; AfterIndex < values.length; AfterIndex++){
        		newValues[repList.size()+AfterIndex] = values[AfterIndex];
        	}
     
        	this.values = newValues;
        }
        
        public int size(){
        	return this.values.length;
        } 
        
        public static void main(String[] args) { 
                Vector first = new Vector(1, 2, 3, 4, 5);
                Vector second = new Vector(6, 7, 8, 9, 10);
                List<Double> weights = Arrays.asList(0.0 , 0.0 , 0.0 , 0.5 , 0.5);
                System.out.println(Vector.euclideanNorm(first, second,weights));
                System.out.println(Vector.calculateCenter(first, second));
                List<Double> repList =  Arrays.asList(1.0, 0.0 , 0.0);
                first.replaceValueAtIndex(2, repList);
                System.out.println(first.toString());
        }
        
        public static Vector calculateCenter( Vector... vectors) {
                if (vectors == null || vectors.length == 0)
                        throw new IllegalArgumentException();
                
                Vector center = new Vector(Arrays.copyOf(vectors[0].values, vectors[0].values.length));
                for (int i = 1; i < vectors.length; i++) {
                        if (vectors[i].values.length != center.values.length)
                                throw new IllegalArgumentException("vector " + i + "'s dimension is not compatible with first vector!");
                        
                        for (int j = 0; j < center.values.length; j++) {
                                center.values[j] += vectors[i].values[j]; 
                        }
                }
                
                for (int i = 0; i < center.values.length; i++) {
                        center.values[i] = center.values[i] / vectors.length; 
                }
                
                return center;
                
        }
        
}