package dummyEncodingLibrary;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import WeightedKmeans.Vector;

public class DummyEncoding {

	/**
	 * int,  map<int, double>
	 * the first int is the index in the original data
	 * the second int, which is in the map, is the index of the dummyTable
	 * double d is if the column c1 is value d, it should be true in the column c2
	 */
	private HashMap<Integer, HashMap<Integer, Double>> encoding;
	// This matrix mantains the mapping orginal matrix to encoded matrix
	// Ths mapping facilated easy decoding.
	private HashMap<Integer, ArrayList<Integer>> matrixMapping;
	public double[][] orginalMatrix;
	private boolean[] encodedValueArray;

	/**
	 * 
	 * @param entityMatrix
	 *            - this example-by-feature matrix the rows contributed example
	 *            and the colunms constitute
	 * @param labels
	 */
	public DummyEncoding(double[][] entityMatrix, boolean[] encodeValue) {
		this.orginalMatrix = entityMatrix;
		this.encodedValueArray = encodeValue;
		encoding = new HashMap<Integer, HashMap<Integer, Double>>();
		int base = 0;
		for (int columnIndex = 0; columnIndex < entityMatrix[0].length; columnIndex++) {
			if (encodeValue[columnIndex]) {
				HashMap encodedColumn = getEncoding(entityMatrix, columnIndex,
						base);
				this.encoding.put(columnIndex, encodedColumn);
				base += encoding.get(columnIndex).size();
			} else {
				HashMap<Integer, Double> map = new HashMap<Integer, Double>();
				map.put(base, null);
				this.encoding.put(columnIndex, map);
				base += 1;
			}
		}

	}

	private HashMap<Integer, Double> getEncoding(double[][] entityMatrix,
			int index, int base) {
		HashMap<Integer, Double> map = new HashMap<Integer, Double>();
		ArrayList<Double> valuesSeen = new ArrayList<Double>();
		int count = base;
		for (int rowIndex = 0; rowIndex < entityMatrix.length; rowIndex++) {
			double value = entityMatrix[rowIndex][index];
			if (!valuesSeen.contains(value)) {
				valuesSeen.add(value);
				map.put(base, value);
				base++;
			}
		}

		return map;
	}

	/**
	 * 
	 * @param dummyEncodeMatrix
	 *            - This a example-by-encoded feature matrix
	 * @return returns and decode matrix example b feature matrix.
	 */

	public double[][] getEncodedMatrix() {

		int totalLengthOfEncodeMatrix = 0;
		for (int key : encoding.keySet()) {
			totalLengthOfEncodeMatrix += encoding.get(key).size();

		}
		int orginalColumnLenght = this.orginalMatrix[0].length;
		// Calculate the size of the newMatrix
		int newColumnLength = totalLengthOfEncodeMatrix;
		int newRowLength = this.orginalMatrix.length;

		double[][] encodedArray = new double[newRowLength][newColumnLength];
		for (int row = 0; row < newRowLength; row++) {

			for (int col = 0; col < orginalColumnLenght; col++) {
				if (this.encodedValueArray[col]) {
					HashMap<Integer, Double> map = this.encoding.get(col);
					for (int key : map.keySet()) {
						if (this.orginalMatrix[row][col] == map.get(key)) {
							encodedArray[row][key] = 1;
							break;
						}
					}

				} else {
					encodedArray[row][(Integer) this.encoding.get(col).keySet().toArray()[0]] = this.orginalMatrix[row][col];
				}
			}
		}
		return encodedArray;
	}

	public double[][] decodeMatrix(double[][] dummyEncodeMatrix) {
		int newFeatureLength = encoding.keySet().size();
		int numberOfPatterns = dummyEncodeMatrix.length;
		double[][] decodedMatrix = new double[numberOfPatterns][newFeatureLength];
		for (int row = 0; row < numberOfPatterns; row++) {
			for(int column : this.encoding.keySet()){
				if(this.encodedValueArray[column]){
					HashMap<Integer, Double> map = this.encoding.get(column);
					double largest = 0;
					boolean valueSet=false;
					int index =0;
					for(int key : map.keySet()){
						double value = dummyEncodeMatrix[row][key];
						if(value>largest | !valueSet){
							largest =value;
							index = key;
							valueSet=true;
						}
						decodedMatrix[row][column] = map.get(index);	
					}
				}
				else{
					decodedMatrix[row][column] = dummyEncodeMatrix[row][(Integer) this.encoding.get(column).keySet()
					            										.toArray()[0]];
				}
				
				
			}
		}
		
		return decodedMatrix;
	}

	public List<Double> ExtendHeadings(List<Double> weights) {
		List<Double> newHeadings = new ArrayList<Double>();
		for (int key : encoding.keySet()) {
			int extension = encoding.get(key).keySet().size();
			for(int i =0; i<extension; i++){
				newHeadings.add(weights.get(key));
			}

		}
		return newHeadings;
	}
	
	
	public static void main(String[] args){
		double[][] values = generateTestArray(2,4);
		boolean[] nom =generateNominalEncoding(4);
		DummyEncoding DE = new DummyEncoding(values,nom);
		double[][] encode = DE.getEncodedMatrix();
		double[][] decode = DE.decodeMatrix(encode);
		System.out.println(Arrays.toString(nom));
		System.out.println(Arrays.deepToString(values));
		System.out.println(Arrays.deepToString(encode));
		System.out.println(Arrays.deepToString(decode));
	}
	
	

	public static boolean[] generateNominalEncoding(int cols){
		boolean[] noms = new boolean[cols];
		for (int c = 0; c < cols; c++) {
			noms[c] = ((Math.random() * 10)>5) ? false : true;
		}
		return noms;
	}
	
	public static double[][] generateTestArray(int rows, int cols) {
		double[][] a2 = new double[rows][cols];
		double count = 0;
		for (int r = 0; r < rows; r++) {
			for (int c = 0; c < cols; c++) {
				a2[r][c] = (int) (Math.random() * 10);
			}
		}
		return a2;
	}
}
//