package pistis.wizards;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.operation.*;
import java.lang.reflect.InvocationTargetException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.CoreException;
import java.io.*;

import org.eclipse.ui.*;
import org.eclipse.ui.ide.IDE;

import ClusterDriver.SystemController;

/**
 * This is a sample new wizard. Its role is to create a new file 
 * resource in the provided container. If the container resource
 * (a folder or a project) is selected in the workspace 
 * when the wizard is opened, it will accept it as the target
 * container. The wizard creates one file with the extension
 * "mpe". If a sample multi-page editor (also available
 * as a template) is registered for the same extension, it will
 * be able to open it.
 */

public class AnonymousData extends Wizard implements INewWizard {
	private AnonymousDataPage1 page1;
	private AnonymousDataPage2 page2;
	private AnonymousDataPage3 page3;
	private ISelection selection;

	/**
	 * Constructor for AnonymousData.
	 */
	public AnonymousData() {
		super();
		setNeedsProgressMonitor(true);
	}
	
	/**
	 * Adding the page to the wizard.
	 */

	public void addPages() {
		page1 = new AnonymousDataPage1(selection);
		page2 = new AnonymousDataPage2("Database Connection");
		page3 = new AnonymousDataPage3("Weights and Nominal");
		addPage(page1);
		addPage(page2);
		addPage(page3);
	}

	/**
	 * This method is called when 'Finish' button is pressed in
	 * the wizard. We will create an operation and run it
	 * using wizard as execution context.
	 */
	public boolean performFinish() {
		System.out.println("************* performFinish called BL.");
		final String containerName = page1.getContainerName();
		final String connectionString = page2.getConnectionStringInfo().getText();
		final String username = page2.getUsernameInfo().getText();
		final String password = page2.getPasswordInfo().getText();
		final String database = page2.getDatabaseInfo().getText();
		final String query = page2.getQueryInfo().getText();
		final String[] numClusterArray = (page3.getClusterInfo().getText()).split("~");
		final double support = 0.01;//Double.parseDouble(page3.getSupportInfo().toString());
		final double confidence = 0.95;//Double.parseDouble(page3.getConfidenceInfo().toString());
		final String filenameWeights = page3.getWeightsSelection().getText();
		final String filenameBoolean = page3.getNominalSelection().getText();
		final int maxCandidates =1000;
		
		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					runAnalysis(containerName, monitor, connectionString, database,  username, password, query, numClusterArray, filenameWeights, filenameBoolean,  support, confidence, maxCandidates);			
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		try {
			getContainer().run(true, false, op);
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			return false;
		}
		return true;
	}
	
	
	
	/**
	 * The method that runs the clustering and association rule mining approach. 
	 * @param connectionString 
	 * @param database 
	 * @param password 
	 * @param query 
	 * @param numClusterArray 
	 * @param filenameWeights 
	 * @param filenameBoolean 
	 * @param outputLocation 
	 * @param support 
	 * @param confidence 
	 */
	private void runAnalysis(String containerName, IProgressMonitor monitor, String connectionString, String database,String username, String password, String query, String[] numClusterArray, String filenameWeights, String filenameBoolean, double support, double confidence ,  int maxCandidates )
		throws CoreException {
		// create a sample file
		monitor.beginTask("Begining Analysis", 2);
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IResource resource = root.findMember(new Path(containerName));
		if (!resource.exists() || !(resource instanceof IContainer)) {
			throwCoreException("Container \"" + containerName + "\" does not exist.");
		}
		IContainer container = (IContainer) resource;
		String outputLocation = container.getLocation().toOSString();
		try {
			SystemController.RunClustering(connectionString, database, username, password, query, numClusterArray, filenameWeights, filenameBoolean, outputLocation, support, confidence, maxCandidates);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		monitor.worked(2);
	}
	
	
	
	
	
	
	
	/**
	 * We will initialize file contents with a sample text.
	 */

	private InputStream openContentStream() {
		String contents =
			"This is the initial file contents for *.mpe file that should be word-sorted in the Preview page of the multi-page editor";
		return new ByteArrayInputStream(contents.getBytes());
	}

	private void throwCoreException(String message) throws CoreException {
		IStatus status =
			new Status(IStatus.ERROR, "PISTIS", IStatus.OK, message, null);
		throw new CoreException(status);
	}

	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}
}